using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;

namespace Rasterbater
{
    public enum BackendProviders
    {
        svg,
        eps,
        postscript,
        ps,
        pdf,
        pgm,
        gimppath,
        xfig
    }

    public enum TurnPolicies
    {
        minority,
        left,
        right,
        black,
        white,        
        majority,
        random
    }

    public class PotraceController
    {
        //I/O options
        public string OutputFile;
        public string InputFile;

        //Backend Selection
        public BackendProviders VectorizationProvider = BackendProviders.svg;

        //Algorithm Options
        public TurnPolicies TurnPolicy = TurnPolicies.minority;
        public int TurdSize = 2;
        public double AlphaMax = 1;
        public bool DisableCurveOptimization; //true += '-n'
        public double CurveOptimizationTolerance = 0.2;
        public int UnitsPerPixel = 10;

        //Scaling and placement options
        //Not built in for faster development, maybe add this stuff later
        //It seems kind of pointless since end viewer programs can be used for doing this stuff.
        //public int Width = -1; // < 0 = not provided.
        //public int Height = -1;
        //public int ResolutionX = -1;
        //public int ResolutionY = -1; //Only used if X is > 0
        //public int ScaleX = -1;
        //public int ScaleY = -1; //Only used if X is > 0
        //public float Stretch = -1;
        //public float ClockwiseRotation = -1;
        //public int Margin = -1;
        //public int LeftMargin = -1;
        //public int RightMargin = -1;
        //public int TopMargin = -1;
        //public int BottomMargin = -1;
        
        //Output options
        public string Color = "#000000";
        public string FillColor = string.Empty; //Transparent is default by app
        public bool ConvertWhiteShapresToOpaque = false; //true += --opaque
        public bool GroupRelatedPathsTogether = false; //true += --group

        //Postscript / EPS Options
        public string PageSize = string.Empty; //Empty = not provided.
        public bool DoNotCompressOutput = false; //true += -c
        public bool UsePostscriptLevelTwoCompression = true; //true += --level2
        public bool UsePostscriptLevelThreeCompression = false; //true += --level3
        public bool DoNotOptimizeForFileSize = false; //true += --longcoding

        //PGM options
        public double GammaLevel = 2.2;

        //Fontend Options
        public double BlackLevel = 0.5;
        public bool InvertBitmap = false; //True += --invert


        public const string CommandLineArgsFormat =
            "-o \"{0}\"" // OutputFile
            + " --backend {1}" //VectorizationProvider.ToString()
            + " --turnpolicy {2}" //TurnPolicy.ToString()
            + " --turdsize {3}" //TurdSize
            + " --alphamax {4}" //AlphaMax
            + "{5}" //if(DisableCurveOptimization) " -n"
            + " --opttolerance {6}" //CurveOptimizationTolerance
            + " --unit {7}" //UnitsPerPixel
            + " -C {8}" //Color
            + "{9}" //if(!string.IsNullOrEmpty(FillColor) " --fillcolor " + FillColor
            + "{10}" //if(ConvertWhiteShapresToOpaque) " --opaque"
            + "{11}" //if(GroupRelatedPathsTogether) " --group"
            + "{12}" //if(!string.IsNullOrEmpty(PageSize)) "-P " + PageSize
            + "{13}" //if(DoNotCompressOutput) " --cleartext"
            + "{14}" //if(UsePostscriptLevelTwoCompression) " --level2"
            + "{15}" //if(UsePostscriptLevelThreeCompression) " --level3"
            + "{16}" //if(DoNotOptimizeForFileSize) " --longcoding"
            + " --gamma {17}" //GammaLevel
            + " --blacklevel {18}" //BlackLevel
            + "{19}" //if(InvertBitmap) " --invert"
            + " \"{20}\""; //Input file


        public string PoTraceExecutableFile = Application.StartupPath + @"\potrace\potrace.exe";


        /// <summary>
        /// Gets the command line args.
        /// </summary>
        /// <returns>CommandLineArgsFormat, formatted.</returns>
        public string GetCommandLineArgs()
        {
            return string.Format(CommandLineArgsFormat,
                OutputFile,
                VectorizationProvider.ToString(),
                TurnPolicy.ToString(),
                TurdSize,
                AlphaMax,
                DisableCurveOptimization ? " -n" : string.Empty,
                CurveOptimizationTolerance,
                UnitsPerPixel,
                Color,
                !string.IsNullOrEmpty(FillColor) ? " --fillcolor " + FillColor : string.Empty,
                ConvertWhiteShapresToOpaque ? " --opaque" : string.Empty,
                GroupRelatedPathsTogether ? " --group" : string.Empty,
                !string.IsNullOrEmpty(PageSize) ? "-P " + PageSize : string.Empty,
                DoNotCompressOutput ? " --cleartext" : string.Empty,
                UsePostscriptLevelTwoCompression ? " --level2" : string.Empty,
                UsePostscriptLevelThreeCompression ? " --level3" : string.Empty,
                DoNotOptimizeForFileSize ? " --longcoding" : string.Empty,
                GammaLevel,
                BlackLevel,
                InvertBitmap ? " --invert" : string.Empty,
                InputFile
            );
        }

        public string ExecutePoTrace()
        {
            ProcessStartInfo psi = new ProcessStartInfo();

            psi.Arguments = GetCommandLineArgs();
            psi.FileName = PoTraceExecutableFile;
            psi.UseShellExecute = false;
            psi.RedirectStandardOutput = true;
            psi.RedirectStandardError = true;

            Process proc = Process.Start(psi);

            proc.WaitForExit();

            string result = "\r\nCMDLINE ARGS: " + psi.Arguments + "\r\n";

            string stdIoText = proc.StandardOutput.ReadToEnd();
            string stdIoErr = proc.StandardError.ReadToEnd();

            if (stdIoText.Length == 0 && stdIoErr.Length == 0) stdIoText = "Success."; //Nothing == success.

            result += "\r\nResult:\r\n" + stdIoText;

            if(stdIoErr.Length > 0) result += "\r\nErrors:\r\n" + stdIoErr;

            proc.Dispose();


            return result;
            
        }


        public string ProcessBatch(string[] inputFiles, string outputDirectory)
        {
            //Uses the current running settings and adjusts input / output file names
            //based on provier

            string fileExtension = "." + VectorizationProvider.ToString();
            //In the event I figure out the filename extensions for gimppath / postscript providers are diff
            //I can change that above ^^^^


            //Ensure directory exists
            if (!Directory.Exists(outputDirectory))
            {
                Directory.CreateDirectory(outputDirectory);
            }

            string log = "Batch process:\r\n";

            foreach (string fileName in inputFiles)
            {
                FileInfo fi = new FileInfo(fileName);

                InputFile = fileName;

                OutputFile = outputDirectory + @"\" + fi.Name + fileExtension;

                log += "\r\n\r\nConverting file: " + fileName + "\r\n------\r\n" + ExecutePoTrace();
            }

            return log;
        }
    }
}
